/*-
 * Copyright (c) 2002-2003 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_misc.c,v 1.6 2005/03/20 03:57:50 nork Exp $
 */

#include <sys/param.h>
#include <stddef.h>
#include <stdlib.h>
#include <assert.h>
#include "pluginwrapper.h"


/*
 * This is used by brk(2), sbrk(2) on Linux.  But FreeBSD's ones is different
 * from Linux's ones.  Therefore, this is dummy variable.
 */
#if 1
void *___brk_addr = NULL;		/* optimaization */
#else
void *__curbrk = NULL;
extern FILE  *___brk_addr __attribute__ ( (alias("__curbrk")) );
#endif

/*
 * 
 */
void *
memalign(size_t boundary, size_t size)
{
#if	1
	void *p = malloc(size);

	dprintf("memalign(%u, %u) = %p", boundary, size, p);

	return p;	/* XXX: Almost(maybe) this is good */
#else
	uintptr_t p = (uintptr_t) malloc(size + boundary);

	if(  p  )  {
		p = roundup2(size,  boundary);
	}

	dprintf("memalign(%u, %u) = %lu", boundary, size, p);

	return p;
#endif
}


/*
 * 
 */
void
__assert_fail(const char *assertion, const char *file,
	unsigned int line, const char *function)
{
#if defined(__DragonFly__) || __FreeBSD_version < 500000
	(void)function;
	__assert(file, line, assertion);
#else
	__assert(function, file, line, assertion);
#endif
}
