/*-
 * Copyright (c) 2004-2005 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_ipc.c,v 1.3 2005/01/08 19:17:18 nork Exp $
 */

/*
 *
 */
#include "pluginwrapper.h"
#include "linux.h"
#include <errno.h>
#define	_KERNEL
#include <sys/shm.h>
#undef	_KERNEL

/*
 *
 */
static void
bsd_to_linux_ipc_perm(struct ipc_perm *bpp, struct linux_ipc_perm *lpp)
{
	lpp->key  = bpp->key;
	lpp->uid  = bpp->uid;
	lpp->gid  = bpp->gid;
	lpp->cuid = bpp->cuid;
	lpp->cgid = bpp->cgid;
	lpp->mode = bpp->mode;
	lpp->seq  = bpp->seq;
}

static void
bsd_to_linux_shmid_ds(struct shmid_ds *bsp, struct linux_shmid_ds *lsp)
{
	bsd_to_linux_ipc_perm(&bsp->shm_perm, &lsp->shm_perm);
	lsp->shm_segsz  = bsp->shm_segsz;
	lsp->shm_lpid   = bsp->shm_lpid;
	lsp->shm_cpid   = bsp->shm_cpid;
	lsp->shm_nattch = bsp->shm_nattch;
	lsp->shm_atime  = bsp->shm_atime;
	lsp->shm_dtime  = bsp->shm_dtime;
	lsp->shm_ctime  = bsp->shm_ctime;
	/* this goes (yet) SOS */
	lsp->private3   = (unsigned long)(bsp->shm_internal);
}

static void
linux_to_bsd_ipc_perm(struct linux_ipc_perm *lpp, struct ipc_perm *bpp)
{
	bpp->key  = lpp->key;
	bpp->uid  = lpp->uid;
	bpp->gid  = lpp->gid;
	bpp->cuid = lpp->cuid;
	bpp->cgid = lpp->cgid;
	bpp->mode = lpp->mode;
	bpp->seq  = lpp->seq;
}

static void
linux_to_bsd_shmid_ds(struct linux_shmid_ds *lsp, struct shmid_ds *bsp)
{
	linux_to_bsd_ipc_perm(&lsp->shm_perm, &bsp->shm_perm);
	bsp->shm_segsz    = lsp->shm_segsz;
	bsp->shm_lpid     = lsp->shm_lpid;
	bsp->shm_cpid     = lsp->shm_cpid;
	bsp->shm_nattch   = lsp->shm_nattch;
	bsp->shm_atime    = lsp->shm_atime;
	bsp->shm_dtime    = lsp->shm_dtime;
	bsp->shm_ctime    = lsp->shm_ctime;
	/* this goes (yet) SOS */
	bsp->shm_internal = (void *)(lsp->private3);
}


/*
 * shmat(2) system call
 */
extern void * _shmat(int, const void *, int);

void *
shmat(int shmid, const void *addr, int flag)
{
	void	*ret;

	CLEAR_ERRNO;

	ret = _shmat(shmid, addr, flag);

	dprintf("shmat(shmid=%d, addr=%08x, flag=0x%x) = %08x / ERRNO = %d", shmid, addr, flag, ret, errno);

	return ret;
}

/*
 * shmctl(2) system call
 */
extern int _shmctl(int, int, struct shmid_ds *);

int
shmctl(int shmid, int l_cmd, struct linux_shmid_ds *l_buf)
{
	int	ret;
	struct shmid_ds	buf;

	CLEAR_ERRNO;

	switch(  l_cmd  )  {
	case	LINUX_IPC_RMID:
		if(  l_buf != 0  )
			linux_to_bsd_shmid_ds(l_buf, &buf);
		ret = _shmctl(shmid, IPC_RMID, l_buf != 0 ? &buf : 0);
		break;

	case	LINUX_IPC_STAT:
		ret = _shmctl(shmid, IPC_STAT, &buf);
		if(  ret == 0  )  {
			bsd_to_linux_shmid_ds(&buf, l_buf);
		}
		break;

	default:
		dprintf("Not implemented shctl(2)");
		ret   = -1;
		errno = ENOSYS;
	}

	dprintf("shctl(shmid=%d, l_cmd=%d, l_buf=%08x) = %d / ERRNO = %d", shmid, l_cmd, l_buf, ret, errno);

	return ret;
}
