/*-
 * Copyright (c) 2002-2003 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_misc.c,v 1.4 2005/01/09 20:48:11 nork Exp $
 */

/*
 *
 */
#include "pluginwrapper.h"
#include "linux.h"
#include <errno.h>


/*
 * mkfifo(2) system call
 */
#ifdef DEBUG
extern int _mkfifo(const char *, mode_t);

int
mkfifo(const char *path, mode_t mode)
{
	int ret;

	CLEAR_ERRNO;

	ret = _mkfifo(path, mode);

	dprintf("mkfifo(path=%s, mode=%04o) = %d / ERRNO = %d", path, mode, ret, errno);

	return ret;
}
#endif


/*
 * waitpid(2) system call
 */
#ifdef DEBUG
pid_t _waitpid(pid_t, int *, int);

pid_t
waitpid(pid_t pid, int *status, int options)
{
	pid_t ret;

	CLEAR_ERRNO;

	ret = _waitpid(pid, status, options);

	dprintf("waitpid(%d, 0x%p(->%d), %d) = %d / ERRNO = %d", pid, status, (status ? *status : 0), options, ret, errno);

	return ret;
}
#endif


/*
 * unlink(2) system call
 */
#ifdef DEBUG
extern int _unlink(const char *);

int
unlink(const char *path)
{
	int ret;

	CLEAR_ERRNO;

	ret = _unlink(path);

	dprintf("unlink(path=%s) = %d / ERRNO = %d", path, ret, errno);

	return ret;
}
#endif
