/*-
 * Copyright (c) 2002-2005 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux.h,v 1.10 2005/07/21 18:50:07 nork Exp $
 */

#ifndef	__LINUX_FILE__
#define	__LINUX_FILE__

#include <sys/types.h>


#if	defined(__i386__) || defined(__amd64__)
#define	_LINUX_IOC_NRBITS	8
#define	_LINUX_IOC_TYPEBITS	8
#define	_LINUX_IOC_SIZEBITS	14
#define	_LINUX_IOC_DIRBITS	2
#else
#error	NOT SUPPORT ARCHITECTURE
#endif


#define	_LINUX_IOC_NRMASK	( (1 << _LINUX_IOC_NRBITS  ) - 1 )
#define	_LINUX_IOC_TYPEMASK	( (1 << _LINUX_IOC_TYPEBITS) - 1 )
#define	_LINUX_IOC_SIZEMASK	( (1 << _LINUX_IOC_SIZEBITS) - 1 )
#define	_LINUX_IOC_DIRMASK	( (1 << _LINUX_IOC_DIRBITS ) - 1 )

#define	_LINUX_IOC_NRSHIFT	0
#define	_LINUX_IOC_TYPESHIFT	(_LINUX_IOC_NRSHIFT   + _LINUX_IOC_NRBITS)
#define	_LINUX_IOC_SIZESHIFT	(_LINUX_IOC_TYPESHIFT + _LINUX_IOC_TYPEBITS)
#define	_LINUX_IOC_DIRSHIFT	(_LINUX_IOC_SIZESHIFT + _LINUX_IOC_SIZEBITS)

#define	_LINUX_IOC_NONE		0U
#define	_LINUX_IOC_WRITE	1U
#define	_LINUX_IOC_READ		2U


#define	_LINUX_IOC(dir, type, nr, size)		\
	(  ( (dir)  << _LINUX_IOC_DIRSHIFT  ) |	\
	   ( (type) << _LINUX_IOC_TYPESHIFT ) |	\
	   ( (nr)   << _LINUX_IOC_NRSHIFT   ) |	\
	   ( (size) << _LINUX_IOC_SIZESHIFT )  )

#define	_LINUX_IO(type, nr)			\
		_LINUX_IOC(_LINUX_IOC_NONE,                  (type), (nr), 0)
#define	_LINUX_IOR(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_READ,                  (type), (nr), sizeof(size))
#define	_LINUX_IOW(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_WRITE,                 (type), (nr), sizeof(size))
#define	_LINUX_IOWR(type, nr, size)		\
		_LINUX_IOC(_LINUX_IOC_READ|_LINUX_IOC_WRITE, (type), (nr), sizeof(size))


#define	LINUX_O_RDONLY			00
#define	LINUX_O_WRONLY			01
#define	LINUX_O_RDWR			02
#define	LINUX_O_NDELAY			LINUX_O_NONBLOCK
#define	LINUX_O_APPEND			02000
#define	LINUX_O_SYNC			010000
#define	LINUX_O_NONBLOCK		04000
#define	LINUX_FASYNC			020000
#define	LINUX_O_CREAT			0100
#define	LINUX_O_TRUNC			01000
#define	LINUX_O_EXCL			0200
#define	LINUX_O_NOCTTY			0400

#define	LINUX_F_DUPFD			0
#define	LINUX_F_GETFD			1
#define	LINUX_F_SETFD			2
#define	LINUX_F_GETFL			3
#define	LINUX_F_SETFL			4
#define	LINUX_F_GETLK			5
#define	LINUX_F_SETLK			6
#define	LINUX_F_SETLKW			7
#define	LINUX_F_SETOWN			8
#define	LINUX_F_GETOWN			9

#define	LINUX_AF_UNSPEC			0
#define	LINUX_AF_UNIX			1
#define	LINUX_AF_INET			2
#define	LINUX_AF_AX25			3
#define	LINUX_AF_IPX			4
#define	LINUX_AF_APPLETALK		5
#define	LINUX_AF_INET6			10

#define	LINUX_SO_DEBUG			1
#define	LINUX_SO_REUSEADDR		2
#define	LINUX_SO_TYPE			3
#define	LINUX_SO_ERROR			4
#define	LINUX_SO_DONTROUTE		5
#define	LINUX_SO_BROADCAST		6
#define	LINUX_SO_SNDBUF			7
#define	LINUX_SO_RCVBUF			8
#define	LINUX_SO_KEEPALIVE		9
#define	LINUX_SO_OOBINLINE		10
#define	LINUX_SO_NO_CHECK		11
#define	LINUX_SO_PRIORITY		12
#define	LINUX_SO_LINGER			13

#define	LINUX_IP_TOS			1
#define	LINUX_IP_TTL			2
#define	LINUX_IP_HDRINCL		3
#define	LINUX_IP_OPTIONS		4

#define	LINUX_SOL_SOCKET		1
#define	LINUX_SOL_IP			0
#define	LINUX_SOL_IPX			256
#define	LINUX_SOL_AX25			257
#define	LINUX_SOL_TCP			6
#define	LINUX_SOL_UDP			17

#define	LINUX_IP_MULTICAST_IF		32
#define	LINUX_IP_MULTICAST_TTL		33
#define	LINUX_IP_MULTICAST_LOOP		34
#define	LINUX_IP_ADD_MEMBERSHIP		35
#define	LINUX_IP_DROP_MEMBERSHIP	36

#define	LINUX_IPC_INFO			3
#define	LINUX_SHM_INFO			14
#define	LINUX_IPC_STAT			2
#define	LINUX_SHM_STAT			13
#define	LINUX_IPC_SET			1
#define	LINUX_IPC_RMID			0
#define	LINUX_SHM_LOCK			11
#define	LINUX_SHM_UNLOCK		12

#define	LINUX_SIGHUP			1
#define	LINUX_SIGINT			2
#define	LINUX_SIGQUIT			3
#define	LINUX_SIGILL			4
#define	LINUX_SIGTRAP			5
#define	LINUX_SIGABRT			6
#define	LINUX_SIGIOT			LINUX_SIGABRT
#define	LINUX_SIGBUS			7
#define	LINUX_SIGFPE			8
#define	LINUX_SIGKILL			9
#define	LINUX_SIGUSR1			10
#define	LINUX_SIGSEGV			11
#define	LINUX_SIGUSR2			12
#define	LINUX_SIGPIPE			13
#define	LINUX_SIGALRM			14
#define	LINUX_SIGTERM			15
#define	LINUX_SIGSTKFLT			16
#define	LINUX_SIGCHLD			17
#define	LINUX_SIGCONT			18
#define	LINUX_SIGSTOP			19
#define	LINUX_SIGTSTP			20
#define	LINUX_SIGTTIN			21
#define	LINUX_SIGTTOU			22
#define	LINUX_SIGURG			23
#define	LINUX_SIGXCPU			24
#define	LINUX_SIGXFSZ			25
#define	LINUX_SIGVTALRM			26
#define	LINUX_SIGPROF			27
#define	LINUX_SIGWINCH			28
#define	LINUX_SIGIO			29
#define	LINUX_SIGPOLL			LINUX_SIGIO
#define	LINUX_SIGPWR			30
#define	LINUX_SIGSYS			31
#define	LINUX_SIGTBLSZ			31
#define	LINUX_NSIG			32
#define	LINUX_NSIG_WORDS		2

#define	LINUX_SA_NOCLDSTOP		0x00000001
#define	LINUX_SA_NOCLDWAIT		0x00000002
#define	LINUX_SA_SIGINFO		0x00000004
#define	LINUX_SA_RESTORER		0x04000000
#define	LINUX_SA_ONSTACK		0x08000000
#define	LINUX_SA_RESTART		0x10000000
#define	LINUX_SA_INTERRUPT		0x20000000
#define	LINUX_SA_NOMASK			0x40000000
#define	LINUX_SA_ONESHOT		0x80000000

#define	LINUX_EAGAIN			11

#define LINUX_MAP_SHARED	0x0001
#define LINUX_MAP_PRIVATE	0x0002
#define LINUX_MAP_FIXED		0x0010
#define LINUX_MAP_ANON		0x0020
#define LINUX_MAP_GROWSDOWN	0x0100

#if	0
/*
 * XXX: assume that Linux's 'struct linux_*' are
 *      FreeBSD's one of the same types.
 */

typedef struct linux_audio_buf_info {
	int		 fragments;
	int		 fragstotal;
	int		 fragsize;
	int		 bytes;
} linux_audio_buf_info;

typedef	struct linux_count_info {
	int		 bytes;
	int		 blocks;
	int		 ptr;
} linux_count_info;

#else

#define	linux_audio_buf_info			audio_buf_info
#define	linux_count_info			count_info

#endif


#define	LINUX_SNDCTL_DSP_RESET			_LINUX_IO  ('P',   0)
#define	LINUX_SNDCTL_DSP_SYNC			_LINUX_IO  ('P',   1)
#define	LINUX_SNDCTL_DSP_SETFMT			_LINUX_IOWR('P',   5, int)
#define	LINUX_SNDCTL_DSP_STEREO			_LINUX_IOWR('P',   3, int)
#define	LINUX_SNDCTL_DSP_SPEED			_LINUX_IOWR('P',   2, int)
#define	LINUX_SNDCTL_DSP_CHANNELS		_LINUX_IOWR('P',   6, int)
#define	LINUX_SNDCTL_DSP_SETFRAGMENT		_LINUX_IOWR('P',  10, int)
#define	LINUX_SNDCTL_DSP_GETFMTS		_LINUX_IOR ('P',  11, int)
#define	LINUX_SNDCTL_DSP_GETCAPS		_LINUX_IOR ('P',  15, int)
#define	LINUX_SNDCTL_DSP_GETOPTR		_LINUX_IOR ('P',  18, linux_count_info)
#define	LINUX_SNDCTL_DSP_GETOSPACE		_LINUX_IOR ('P',  12, linux_audio_buf_info)
#define	LINUX_MIXER_READ(dev)			_LINUX_IOR ('M', dev, int)
#define	LINUX_SOUND_MIXER_DEVMASK		0xfe
#define	LINUX_SOUND_MIXER_READ_DEVMASK		LINUX_MIXER_READ(LINUX_SOUND_MIXER_DEVMASK)
#define	LINUX_SOUND_PCM_READ_CHANNELS		_LINUX_IOR ('P',   6, int)
#define	LINUX_SOUND_PCM_WRITE_CHANNELS		LINUX_SNDCTL_DSP_CHANNELS


struct linux_timespec {
	uint32_t		tv_sec;
	uint32_t		tv_nsec;
};


struct linux_stat {
        unsigned short int	st_dev;
        unsigned short int	__pad1;
        uint32_t		st_ino;
        unsigned short int	st_mode;
        unsigned short int	st_nlink;
        unsigned short int	st_uid;
        unsigned short int	st_gid;
        unsigned short int	st_rdev;
        unsigned short int	__pad2;
        uint32_t		st_size;
        uint32_t		st_blksize;
        uint32_t		st_blocks;
        struct linux_timespec	st_atimespec;
        struct linux_timespec	st_mtimespec;
        struct linux_timespec	st_ctimespec;
        uint32_t		__unused4;
        uint32_t		__unused5;
};


typedef struct {
	unsigned int	__bits[LINUX_NSIG_WORDS];
} l_sigset_t;


struct linux_sigaction {
	void (*lsa_handler)(int);
	uint32_t	lsa_mask;
	uint32_t	lsa_flags;
	void (*lsa_restorer)(void);
} l_osigaction_t;

#define	LINUX_SIGEMPTYSET(set)		(set) = 0
#define	LINUX_SIGISMEMBER(set, sig)	(set) & _SIG_BIT(sig)
#define	LINUX_SIGADDSET(set, sig)	(set) |= _SIG_BIT(sig)


typedef struct linux_pthread_mutex {
	int32_t dummy[6];
} linux_pthread_mutex_t;

typedef struct linux_pthread_mutexattr {
	int32_t dummy;
} linux_pthread_mutexattr_t;

struct linux_ipc_perm {
	int		key;
	unsigned short	uid;
	unsigned short	gid;
	unsigned short	cuid;
	unsigned short	cgid;
	unsigned short	mode;
	unsigned short	seq;
};

struct linux_shmid_ds {
	struct linux_ipc_perm	shm_perm;
	int			shm_segsz;
	int32_t			shm_atime;
	int32_t			shm_dtime;
	int32_t			shm_ctime;
	unsigned short		shm_cpid;
	unsigned short		shm_lpid;
	short			shm_nattch;
	unsigned short		private1;
	unsigned long		private2;
	unsigned long		private3;
};

struct linux_sockaddr {
	unsigned short	sa_family;
	char		sa_data[14];
};

#endif	/* __LINUX_FILE__ */
